#!/usr/bin/env python3
# Copyright (C) 2025 Spearhead Systems SRL

import datetime
from cmk.base.plugins.agent_based.agent_based_api.v1 import register, Result, Service, State


# Incoming agent output takes the form:
#
# google.com:
# 2028-09-14
# yahoo.com:
# 2026-01-19
# 
# The domains being queries end with a colon. The expiry response does not.
#
# Return a dictionary which uses domain as key, and returns an expiry as
# datetime.date.
def parse_domains_expiry(string_table):
    expiries = {}
    current_domain = ""

    for line in string_table:
        line = line[0]

        domain = line[:-1]
        ending = line[-1]

        if ending == ":":
            current_domain = domain
        else:
            date = datetime.datetime.strptime(line, '%Y-%m-%d').date()
            expiries[current_domain] = date

    return expiries


def discover_domains_expiry(section):
    for domain, date in section.items():
        yield Service(item=domain)


def check_domains_expiry(item, params, section):
    expiry = section.get(item)
    if not expiry:
        yield Result(state=State.WARN, summary="Expiry not found in whois")
        return

    alert_delta = params.get("days_remaining")
    if not alert_delta:
        yield Result(state=State.WARN, summary="No expiry check rule configured")
        return

    valid_days_left = (expiry - datetime.date.today()).days
    summary = "Domain expires in %s days" % valid_days_left
    state = State.OK

    if valid_days_left < alert_delta[1]:
        state = State.CRIT
    elif valid_days_left < alert_delta[0]:
        state = State.WARN

    yield Result(state=state, summary=summary)


register.agent_section(
    name="domains_expiry",
    parse_function=parse_domains_expiry
)

register.check_plugin(
    name="domains_expiry",
    service_name="Expiry for Domain '%s'",

    discovery_function=discover_domains_expiry,

    check_function=check_domains_expiry,
    check_default_parameters={},
    check_ruleset_name="domains_expiry",
)
