#!/usr/bin/env python3
# Copyright (C) 2025 Spearhead Systems SRL

from cmk.base.plugins.agent_based.agent_based_api.v1 import register, Result, Service, State


# Incoming agent output takes the form:
# 
# google.com:
# ns3.google.com.
# ns1.google.com.
# ns4.google.com.
# ns2.google.com.
# yahoo.com:
# ns2.yahoo.com.
# ns1.yahoo.com.
# ns3.yahoo.com.
# ns4.yahoo.com.
# ns5.yahoo.com.
#
# The domains being queries end with a colon. The nameserver responses end with
# a period. Nameservers follow the domain being queried.
#
# Return a dictionary which uses domain as key, and returns a list of associated
# nameservers for that key.
def parse_domains_nameservers(string_table):
    nameservers = {}
    current_domain = ""

    for line in string_table:
        domain = line[0][:-1]
        ending = line[0][-1]

        if ending == ":":
            current_domain = domain
            nameservers[current_domain] = []
        elif ending == ".":
            nameservers[current_domain].append(domain)

    return nameservers


def discover_domains_nameservers(section):
    for domain, nameservers in section.items():
        yield Service(item=domain)


def check_domains_nameservers(item, params, section):
    nameservers = section.get(item)
    if not nameservers:
        yield Result(state=State.WARN, summary="Nameservers missing")
        return

    configs = params.get("domain_nameservers")
    if not configs:
        yield Result(state=State.WARN, summary="No nameservers check rule configured")
        return

    alert_level = params["alert_level"]

    nameservers.sort()

    domain_found = False

    for config in configs:
        domains = config["domains"]

        if item in domains:
            domain_found = True

            expected_nameservers = config["nameservers"]
            expected_nameservers.sort()

            if expected_nameservers == nameservers:
                yield Result(state=State.OK, summary="Expected nameservers present")
            else:
                expected_str = ", ".join(expected_nameservers)
                found_str    = ", ".join(nameservers)
                yield Result(state=State(alert_level), summary="Mismatch in nameservers. Expected: [%s], Found: [%s]" % (expected_str, found_str))

    if not domain_found:
        yield Result(state=State.WARN, summary="Domain not found in any nameserver check rule")
         


register.agent_section(
    name="domains_nameservers",
    parse_function=parse_domains_nameservers
)

register.check_plugin(
    name="domains_nameservers",
    service_name="Nameservers for Domain '%s'",

    discovery_function=discover_domains_nameservers,

    check_function=check_domains_nameservers,
    check_default_parameters={},
    check_ruleset_name="domains_nameservers",
)
